import { DateHelper, PresetManager, SchedulerConfig } from "@bryntum/scheduler";
import type { AjaxStore } from "@bryntum/scheduler";
import type { UserResourceModel } from "../models/UserResourceModel";
import customViewPresets from "./schedule/customViewPresets";
import eventEditor from "./schedule/eventEditor";
import tbar from "./schedule/tbar";
import {
    CONSTS,
    doNotLetUserChangeType,
    correctEditorIfByRoom,
    hydrateFieldsWithHalActions,
    submittedNewWishWithTheSingleWishTypeAvailable,

} from "./schedule/utils";

export const useScheduleConfig = (
    scheduleId: number,
    isAdmin: boolean,
    userId: number,
    rowKey: string,
    wishTypes: AjaxStore,
    rooms: AjaxStore,
    users: AjaxStore,
): Partial<SchedulerConfig> => {
    return {
        minHeight: 35,
        appendTo: "schedule-calendar",
        tbar: tbar,
        weekStartDay: DateHelper.weekStartDay,
        zoomOnMouseWheel: true,
        zoomOnTimeAxisDoubleClick: false,
        infiniteScroll: false,
        useInitialAnimation: 'slide-from-left',
        viewPreset: customViewPresets.JPPOLITIKENHUS.id,
        presets: [...PresetManager.records, ...Object.values(customViewPresets),],
        syncMask: 'Loading..',
        loadMask: 'Loading..',
        loadMaskError: false,
        createEventOnDblClick: true,
        fillTicks: true,
        snap: true,
        barMargin: 0,
        features: {
            eventMenu: {
                disabled: !isAdmin,
            },
            scheduleMenu: {
                disabled: !isAdmin,
            },
            stripe: true,
            headerZoom: true,
            sort: 'list_order',
            columnPicker: {
                disabled: !isAdmin,
            },
            scheduleTooltip: false,
            dependencies: false,
            eventTooltip: {
                align: 'l-r',
                template: data => `<dl>
                    <dd>${DateHelper.format(data.eventRecord.startDate, 'LT')} - ${DateHelper.format(data.eventRecord.endDate, 'LT')}</dd>
                </dl>`
            },
            eventDrag: {
                disabled: !isAdmin,
                constrainDragToTimeline: false, // Allow drag outside of this Scheduler
            },
            eventDragCreate: false,
            eventDragSelect: isAdmin,
            eventResize: {
                disabled: true,
            },
            eventCopyPaste: {
                disabled: !isAdmin,
                generateNewName: (event) => event.name
            },
            timeRanges: {
                showCurrentTimeLine: true,
                showHeaderElements: true,
                enableResizing: false,
                tooltipTemplate: ({ timeRange }) => timeRange.name
            },
            eventEdit: eventEditor(wishTypes, rooms, users, !isAdmin, rowKey),
        },

        listeners: {
            beforeEventEdit: ({ source, resourceRecord, eventRecord }) => {
                // Nobodies may only create wishes and edit own shifts.
                const isNewEvent = eventRecord.hasGeneratedId
                const isWish = eventRecord.eventType === CONSTS.wish // TODO Swaggerize CONSTS
                const clickingOnSelfByUser = rowKey === "user_id" && eventRecord.resourceId === userId
                const clickingOnSelfByRoom = rowKey === "room_id" && (eventRecord.userId === userId)
                const clickingOnAbsenceByRoom = rowKey === "room_id" && (eventRecord.resourceId === 0)
                const self = clickingOnSelfByUser || clickingOnSelfByRoom || clickingOnAbsenceByRoom
                const showEvent = isAdmin || (self && isNewEvent && isWish) || (self && !isNewEvent && !isWish)
                // if we decide not to show the event but we created a proxy, delete it
                if (!showEvent && isNewEvent) {
                    source.eventStore.remove(eventRecord, true)
                }
                if (!showEvent) {
                    return false
                }
                if (submittedNewWishWithTheSingleWishTypeAvailable(source, wishTypes, resourceRecord, eventRecord, userId, rowKey, isAdmin)) {
                    return false
                }
            },
            beforeEventEditShow: ({ source, editor, eventRecord }) => {
                correctEditorIfByRoom(source, editor, eventRecord, rowKey, userId)
                doNotLetUserChangeType(source, editor, eventRecord)
                // TODO: Set to 8 hours if new event && SHIFT spanning one day
                // TODO: Set to full day if new event && WISH spanning one day
                hydrateFieldsWithHalActions(source, editor, eventRecord, true, scheduleId)
            },
        },

        columns: [
            { // TODO: Insead of icons then just have color?
                type: "resourceInfo",
                text: "Name", // TODO: Translate
                editor: null,
                field: "name",
                showImage: false,
                width: 250,
                locked: true,
                showEventCount: false,
                enableCellContextMenu: false,
                showMeta: (m: UserResourceModel) => isAdmin && rowKey === "user_id" ? `<span style="${Math.abs(m.hours_in_schedule - m.norm_hours) > 8 ? 'color:red;font-weight:bolder;' : ''}">${m.hours_in_schedule} of ${m.norm_hours}</span>` : ``
            },
            {
                text: "List order",
                field: "list_order",
                hidden: true,
                cellCls: 'd-stat-cell',
                enableCellContextMenu: false,
                cls: 'd-stat-cell',
                align: 'center'
            }, // TODO Translate
            {
                text: "Norm hours",
                field: "norm_hours",
                editor: null,
                hidden: true,
                enableCellContextMenu: false,
                cellCls: 'd-stat-cell',
                cls: 'd-stat-cell',
                align: 'center'
            }, // TODO Translate
            {
                text: "Norm weeks",
                field: "norm_weeks",
                editor: null,
                hidden: true,
                enableCellContextMenu: false,
                cellCls: 'd-stat-cell',
                cls: 'd-stat-cell',
                align: 'center'
            }, // TODO Translate
            {
                text: "Days in month",
                field: "days_in_month",
                editor: null,
                hidden: true,
                enableCellContextMenu: false,
                cellCls: 'd-stat-cell',
                cls: 'd-stat-cell',
                align: 'center'
            }, // TODO Translate
            {
                text: "Hours in schedule",
                field: "hours_in_schedule",
                editor: null,
                hidden: true,
                enableCellContextMenu: false,
                cellCls: 'd-stat-cell',
                cls: 'd-stat-cell',
                align: 'center'
            }, // TODO Translate
            {
                text: "Shifts in schedule",
                field: "shifts_in_schedule",
                editor: null,
                hidden: true,
                enableCellContextMenu: false,
                cellCls: 'd-stat-cell',
                cls: 'd-stat-cell',
                align: 'center'
            }, // TODO Translate
            {
                text: "Optional shifts",
                field: "optional_shifts_in_schedule",
                editor: null,
                hidden: true,
                enableCellContextMenu: false,
                cellCls: 'd-stat-cell',
                cls: 'd-stat-cell',
                align: 'center'
            }, // TODO Translate
            {
                text: "Weekends in schedule",
                field: "weekends_in_schedule",
                editor: null,
                hidden: true,
                enableCellContextMenu: false,
                cellCls: 'd-stat-cell',
                cls: 'd-stat-cell',
                align: 'center'
            }, // TODO Translate
            {
                text: "Carry over",
                field: "carry_over",
                editor: null,
                hidden: true,
                enableCellContextMenu: false,
                cellCls: 'd-stat-cell',
                cls: 'd-stat-cell',
                align: 'center'
            }, // TODO Translate
        ],
    }
}