import {AjaxStore, DateHelper, SchedulerConfig, StringHelper, Toast} from "@bryntum/scheduler";
import {WeekTemplateModel} from "../models/WeekTemplateModel";

const rooms = new AjaxStore({
    id: 'rooms',
})

export const useWeekTemplateConfig = (startDate: Date, endDate: Date, teamId: number): Partial<SchedulerConfig> => {
    return {
        ref: "schedulerRef",
        insertFirst: "plan-detail-view",
        eventStyle: 'border',
        fillTicks: true,
        syncMask: null,
        loadMask: null,
        loadMaskError: false,
        snapRelativeToEventStartDate: true,
        flex: 1,
        tbar: [
            {
                ref: "updateWeekCount",
                type: "number",
                placeholder: "Number of weeks",
                label: "Weeks",
                tooltip: "Enter number of weeks",
                value: 1,
                width: 200,
            },
        ],
        crudManager: {
            resourceStore: {
                fields: ['id', 'name', 'list_order', 'color']
            },
            eventStore: {
                modelClass: WeekTemplateModel,
            },
            stores: [rooms,],
            autoLoad: true,
            autoSync: true,
            writeAllFields: true,
            transport: {
                load: {
                    url: `/api/scheduler/week-templates/${teamId}`,
                    method: 'GET',
                },
                sync: {
                    url: `/api/scheduler/week-templates/${teamId}`,
                    method: 'POST',
                    headers: {
                        'Content-Type': 'application/json',
                        'X-XSRF-TOKEN': decodeURIComponent(document.cookie.split(";").find(x => x.includes("XSRF")).trim().replace("XSRF-TOKEN=", ""))
                    }
                }
            },
            validateResponse: true,
            onRequestFail: ({source, requestType, response, responseText, responseOptions}) => {
                source.revertChanges() // Stop retrying
                const responseTextObj = JSON.parse(responseText)
                let message = '';
                if('message' in responseTextObj && 'errors' in responseTextObj) {
                    message = `${responseTextObj.message}: ${JSON.stringify(responseTextObj.errors)}`
                }
                else if('message' in responseTextObj) {
                    message = `${responseTextObj.message}`
                } else {
                    message = JSON.stringify(responseTextObj)
                }
                Toast.show({
                    html: message,
                    color: 'b-red',
                    style: 'color:white',
                    timeout: 5000
                });
            },
        },
        startDate: startDate, // any given Monday
        endDate: endDate, // ..and a subsequent Sunday
        weekStartDay: 1, // TODO: One day this can be a department setting
        zoomOnMouseWheel: false,
        zoomOnTimeAxisDoubleClick: false,

        // This view should be nailed tightly so we create just a single preset and disable zooming
        // https://www.bryntum.com/docs/scheduler-pro/api/Scheduler/preset/PresetManager
        useInitialAnimation: 'slide-from-left',
        viewPreset: "planDetailPreset",
        presets: [
            {
                id: "planDetailPreset",
                name: "planDetailPreset",
                //displayDateFormat : 'HH:mm',
                defaultSpan: 7, // By default, if no end date is supplied to a view it will show 12 hours
                timeResolution: {
                    unit: "day", // Snap dates to intervals of
                    increment: 1,
                },
                headers: [
                    { unit: "week", dateFormat: "W", },
                    { unit: "day", dateFormat: "ddd", },
                ],
            },
        ],
        createEventOnDblClick: {
            useEventModelDefaults: true, // Because WeekTemplateModel has duration 8 and durationUnit "hour"
        },
        features: {
            stripe: true,
            sort: 'list_order',
            scheduleTooltip: false,
            dependencies: false,
            eventTooltip: {
                align: 'l-r',
                template: data => `<dl>
                    <dd>
                    ${data.eventRecord.name}: ${DateHelper.format(data.eventRecord.startDate, 'LT')} - ${DateHelper.format(data.eventRecord.endDate, 'LT')} (${data.eventRecord.duration} ${data.eventRecord.durationUnit})
                    </dd>
                </dl>`
            },
            eventCopyPaste: {
                generateNewName: (event) => event.name
            },
            eventDragSelect: true,
            eventDragCreate: false,
            eventResize: {
                disabled: true,
            },
            timeRanges: false,
            eventEdit: {
                timeFormat: "HH:mm",
                items: {
                    resourceField: {
                        type: "resourcecombo",
                        required: true,
                        validateOnInput: false,
                        label: "L{Resource}",
                        name: "resource",
                        multiSelect: false,
                        editable: true,
                        valueField: "id",
                        displayField: "name",
                        highlightExternalChange: false,
                        destroyStore: true,
                        weight: 200,
                    },
                    roomField: {
                        label: "Room", // TODO: Translate..
                        required: true,
                        validateOnInput: false,
                        name: "room_id", // NOTE: Make sure your EventModel has this field for this to link up correctly
                        type: 'combo',
                        ref: 'roomCombo',
                        store: rooms,
                        placeholder: 'Select room...',
                        weight: 130,
                        valueField: 'id',
                        displayField: 'name',
                    },
                    startDateField: null,
                    startTimeField: {
                        required: true,
                        validateOnInput: false,
                        keepDate: true,
                    },
                    endDateField: null,
                    endTimeField: {
                        required: true,
                        validateOnInput: false,
                        keepDate: true,
                    },
                },
            },
        },
        eventBodyTemplate: data => StringHelper.xss`
        <section>
        <div style="font-size: 0.8em;">${data.headerText}</div>
        <div style="font-weight: 400;">${data.footerText}</div>
        </section>`,

        eventRenderer({eventRecord, resourceRecord, renderData}) {
            renderData.style = `background-color: ${eventRecord.color}`
            return {
                headerText: `${eventRecord.duration} ${eventRecord.durationUnit}`,
                footerText: eventRecord.name || ''
            };
        },
        columns: [
            { // TODO: Insead of icons then just have color?
                type: "resourceInfo",
                text: "Name", // TODO: Translate
                editor: null,
                field: "name",
                showImage: false,
                showEventCount: false,
                width: 250,
                locked: true,
                enableCellContextMenu: false,
            },
            {
                text: "List order", // TODO: Translate
                field: "list_order",
                hidden: true,
                align: 'center',
                enableCellContextMenu: false,
            },
            {
                text: 'Nbr shifts', // TODO: Translate
                editor: null,
                renderer: data => `${data.record.events.length || ''}`,
                align: 'center',
                hidden: true,
                enableCellContextMenu: false,
                sortable: (a, b) =>
                    a.events.length < b.events.length ? -1 : 1,
            }
        ],
    }
}