import { AjaxStore, DateHelper, SchedulerConfig, Toast, PresetManager } from "@bryntum/scheduler";
import { ScheduleItemModelByUser } from "../models/ScheduleItemModelByUser";
import { hydrateFieldsWithHalActions } from "./schedule/utils";
import { PermissionEnum } from "../../../autogenerated";
import customViewPresets from "./schedule/customViewPresets";

const wishTypes = new AjaxStore({
    id: 'wishTypes',
})

const today = DateHelper.clearTime(new Date())

export const useWishesConfig = (teamId: number, permissions: PermissionEnum[], ownUserId: number): Partial<SchedulerConfig> => {
    return {
        ref: "wishRef",
        insertFirst: "wishes-view",
        eventStyle: 'border',
        flex: 1,
        tbar: [
            {
                ref: "scrollTo",
                label: 'Scroll to date',
                inputWidth: '7em',
                width: 'auto',
                type: 'datefield',
                picker: {
                    showWeekColumn: true,
                },
                value: today,
                step: '1d',
                highlightExternalChange: false
            },
        ],
        crudManager: {
            resourceStore: {
                fields: ['id', 'name', 'list_order', 'color']
            },
            eventStore: {
                modelClass: ScheduleItemModelByUser,
            },
            stores: [wishTypes,],
            autoLoad: true,
            autoSync: true,
            writeAllFields: true,
            transport: {
                load: {
                    url: `/api/scheduler/wishes/${teamId}`,
                    method: 'GET',
                },
                sync: {
                    url: `/api/scheduler/wishes/${teamId}`,
                    method: 'POST',
                    headers: {
                        'Content-Type': 'application/json',
                        'X-XSRF-TOKEN': decodeURIComponent(document.cookie.split(";").find(x => x.includes("XSRF")).trim().replace("XSRF-TOKEN=", ""))
                    }
                }
            },
            validateResponse: true, // TODO: Not in PRODUCTION!
            onRequestFail: ({ source, requestType, response, responseText, responseOptions }) => {
                source.revertChanges() // Stop retrying
                const responseTextObj = JSON.parse(responseText)
                let message = '';
                if ('message' in responseTextObj && 'errors' in responseTextObj) {
                    message = `${responseTextObj.message}: ${JSON.stringify(responseTextObj.errors)}`
                }
                else if ('message' in responseTextObj) {
                    message = `${responseTextObj.message}`
                } else {
                    message = JSON.stringify(responseTextObj)
                }
                Toast.show({
                    html: message,
                    color: 'b-red',
                    style: 'color:white',
                    timeout: 5000
                });
            },
        },
        rowHeight: 35, // // If you need changes https://bryntum.com/examples/scheduler/rowheight/
        weekStartDay: 1, // TODO: One day this can be a department setting
        zoomOnMouseWheel: true,
        zoomOnTimeAxisDoubleClick: false,
        infiniteScroll: true,
        useInitialAnimation: 'slide-from-left',
        viewPreset: customViewPresets.JPPOLITIKENHUS.id,
        presets: [...PresetManager.records, ...Object.values(customViewPresets),],
        syncMask: null,
        loadMask: null,
        loadMaskError: false,
        createEventOnDblClick: true,
        listeners: {
            beforeEventEdit: ({ eventRecord, source }) => {
                const cannotWishForOthers: boolean = !permissions.includes(PermissionEnum.ControlWishes)
                const userExistsInStore = source.resourceStore.includes(ownUserId)
                if (cannotWishForOthers && userExistsInStore)
                    eventRecord.resourceId = ownUserId
                if (cannotWishForOthers && !userExistsInStore)
                    return false;
                const isNewWish: boolean = !eventRecord.id?.startsWith("WISHID") ?? false // TODO: Enumize
                if (isNewWish) {
                    const newStartDate = new Date(eventRecord.startDate)
                    newStartDate.setHours(0, 0, 0)
                    const newEndDate = new Date(eventRecord.startDate) // So it is same day..
                    newEndDate.setHours(23, 59, 59)
                    eventRecord.startDate = newStartDate
                    eventRecord.endDate = newEndDate
                }
            },
            beforeEventEditShow: ({ source, editor, eventRecord }) => {
                hydrateFieldsWithHalActions(source, editor, eventRecord, true, 0)
            },
        },
        features: {
            eventMenu: {
                disabled: !permissions.includes(PermissionEnum.ControlWishes),
            },
            scheduleMenu: {
                disabled: !permissions.includes(PermissionEnum.ControlWishes),
            },
            stripe: true,
            sort: 'list_order',
            scheduleTooltip: false,
            dependencies: {
                disabled: true,
            },
            eventTooltip: {
                align: 'l-r',
                template: data => `<dl>
                    <dd>
                    ${Math.round(data.eventRecord.duration)} ${data.eventRecord.durationUnit} (${DateHelper.format(data.eventRecord.startDate, 'LL')} - ${DateHelper.format(data.eventRecord.endDate, 'LL')})
                    </dd>
                </dl>`
            },
            eventCopyPaste: {
                disabled: !permissions.includes(PermissionEnum.ControlWishes),
                generateNewName: (event) => event.name
            },
            eventDragSelect: permissions.includes(PermissionEnum.ControlWishes),
            eventDragCreate: false,
            eventResize: {
                disabled: true,
            },
            timeRanges: false,
            eventEdit: {
                timeFormat: "HH:mm",
                items: {
                    nameField: null,
                    resourceField: {
                        required: true,
                        disabled: !permissions.includes(PermissionEnum.ControlWishes),
                        validateOnInput: false,
                        type: "resourcecombo",
                        label: "L{Resource}",
                        name: "resource",
                        multiSelect: false,
                        editable: true,
                        valueField: "id",
                        displayField: "name",
                        highlightExternalChange: false,
                        destroyStore: true,
                        weight: 200,
                    },
                    wishTypeField: {
                        required: true,
                        validateOnInput: false,
                        label: "Wishtype", // TODO: Translate..
                        name: "wishTypeId", // NOTE: Make sure your EventModel has this field for this to link up correctly
                        type: 'combo',
                        ref: 'wishTypeCombo',
                        store: wishTypes,
                        placeholder: 'Select wish type...',
                        weight: 130,
                        valueField: 'id',
                        displayField: 'name',
                    },
                    startDateField: {
                        required: true,
                        validateOnInput: false,
                        picker: {
                            showWeekColumn: true,
                        }
                    },
                    endDateField: {
                        required: true,
                        validateOnInput: false,
                        picker: {
                            showWeekColumn: true,
                        }
                    },
                    noteField: {
                        label: "Note", // TODO: Translate..
                        name: "note", // NOTE: Make sure your EventModel has this field for this to link up correctly
                        type: 'textarea',
                        ref: 'noteField',
                        placeholder: 'Note...',
                        weight: 800,
                    }
                },
            },
        },

        columns: [
            { // TODO: Insead of icons then just have color?
                type: "resourceInfo",
                text: "Name", // TODO: Translate
                field: "name",
                showImage: false,
                showEventCount: false,
                width: 250,
                locked: true,
                enableCellContextMenu: false,
            },
            {
                text: "List order", // TODO: Translate
                field: "list_order",
                hidden: true,
                align: 'center',
                enableCellContextMenu: false,
            },
            {
                text: 'Nbr tasks', // TODO: Translate
                editor: false,
                renderer: data => `${data.record.events.length || ''}`,
                align: 'center',
                hidden: true,
                sortable: (a, b) => a.events.length < b.events.length ? -1 : 1,
                enableCellContextMenu: false,
            }
        ],
    }
}