import { DateHelper, Popup, Scheduler } from "@bryntum/scheduler";
import type { AjaxStore } from "@bryntum/scheduler";
import type { ScheduleItemModelByUser } from "../../models/ScheduleItemModelByUser";
import type { ActionSpec } from "../../../../autogenerated";
import { AdditionalStoresEnum, EventActionsEnum, EventsEnum, IconsEnum, ShowTextEnum } from "../../eventsEnum"
import { executeHalActionAndTriggerSchedulerEvent, fetchHalActionForScheduleItem } from "../../HAL/base";
import { ScheduleItemModelByRoom } from "../../models/ScheduleItemModelByRoom";

export enum CONSTS {
    historic = "historic",
    shift = "shift",
    wish = "wish",
}

export const pushScheduleActionsToToolBar = (scheduler: Scheduler, scheduleActions: AjaxStore) => {
    const actions: ActionSpec[] = scheduleActions.records
    for (let i = 0; i < actions.length; i++) {
        const action = actions[i]
        const refAndId = action.name

        if (!Object.keys(EventActionsEnum).includes(action.name))
            throw new Error(`Since ${action.name} is not in EventActionsEnum enum we will not be able to emit an event upon success.`)

        scheduler.tbar.add({
            id: refAndId,
            ref: refAndId,
            type: 'button',
            tooltip: action.text,
            icon: IconsEnum[action.name],
            text: ShowTextEnum[action.name] === "true" ? action.text : null,
            onAction: () => {
                executeHalActionAndTriggerSchedulerEvent({
                    scheduler: scheduler,
                    action: action,
                    widgetId: refAndId,
                    onStartEvent: EventsEnum.TbarHalActionLoading,
                    onSuccessEvent: EventActionsEnum[action.name],
                    onFinalEvent: EventsEnum.TbarHalActionStoppedLoading,
                })
            }
        })
    }
}

/** When serving byRoom from the backend, we make the resourceId the room_id. We need to change this for the form */
export const correctEditorIfByRoom = (
    source: Scheduler, editor: Popup, eventRecord: ScheduleItemModelByUser | ScheduleItemModelByRoom, rowKey: string, userId: number
) => {

    const fields: string[] = editor.items.map(i => i.ref)

    if (fields.indexOf("nameField") === -1)
        throw new Error("nameField not in editor. Did you rename it? It should be hidden when grouped by room.")

    if (fields.indexOf("userField") === -1)
        throw new Error("userField not in editor. Did you rename it? It is needed for commit 135ca452")

    if (fields.indexOf("roomField") === -1)
        throw new Error("roomField not in editor. Did you rename it? It is needed for commit 135ca452")

    if (fields.indexOf("wishTypeField") === -1)
        throw new Error("wishTypeField not in editor. Did you rename it? It is needed for commit 135ca452")

    if (rowKey === 'user_id')
        return

    // Rendering editor by room, the name field should be hidden (since always equals username)
    editor.items.find(i => i.ref === "nameField").hide()

    const userField = editor.items.find(i => i.ref === "userField")
    if (userField.value === null)
        userField.value = userId
}

export const doNotLetUserChangeType = (source: Scheduler, editor: Popup, eventRecord: ScheduleItemModelByUser | ScheduleItemModelByRoom) => {
    const fields: string[] = editor.items.map(i => i.ref)
    if (fields.indexOf("eventType") === -1)
        throw new Error("eventType not in editor. Did you rename it?")
    const modelExists = Number.isInteger(eventRecord.backendId) && eventRecord.backendId > 0
    modelExists
        ? editor.items.find(i => i.ref === "eventType").hide()
        : editor.items.find(i => i.ref === "eventType").show()
}

export const hydrateFieldsWithHalActions = (
    source: Scheduler, editor: Popup, eventRecord: ScheduleItemModelByUser | ScheduleItemModelByRoom, disableFieldsWhenAction: boolean
) => {
    const prefix = "PREVIOUSLY_ADDED_BY_DATAPULT_"
    if (eventRecord.id.startsWith("_generated"))
        return true

    editor.remove(editor.items.filter(i => i.ref.startsWith(prefix)))
    fetchHalActionForScheduleItem(eventRecord).then((actions: ActionSpec[]) => {
        actions.forEach(action => {
            const refAndId: string = `${prefix}${eventRecord.id}_${action.name}`
            if (!Object.keys(EventActionsEnum).includes(action.name))
                throw new Error(`Since ${action.name} is not in EventActionsEnum enum we will not be able to emit an event upon success.`)
            editor.add({
                ref: refAndId,
                weight: 1000,
                type: 'button',
                icon: IconsEnum[action.name],
                cls: 'b-transparent',
                text: action.text,
                onAction: () => {
                    executeHalActionAndTriggerSchedulerEvent({
                        scheduler: source,
                        action: action,
                        widgetId: '',
                        onStartEvent: EventsEnum.ScheduleItemHalLoading,
                        onSuccessEvent: EventActionsEnum[action.name],
                        onFinalEvent: EventsEnum.ScheduleItemHalFinal,
                    })
                    editor.close()
                }
            })
        })
    })
}

/**
 * No need to show the popup form for those cases when there is a single wishType (clients like CSG).
 * We return false, if we want the editor to be shown.
 * If you are an admin, you will always get the editor.
 */
export const submittedNewWishWithTheSingleWishTypeAvailable = (
    source: Scheduler, wishTypeStore: AjaxStore, resourceRecord, eventRecord: ScheduleItemModelByUser | ScheduleItemModelByRoom, userId: number, rowKey: string, isAdmin: bool
) => {
    if (isAdmin) { return false }
    if (wishTypeStore === undefined)
        throw new Error("wishTypeStore not found but needed for submitIfNewWishAndJustOneWishType()")
    const onlyOneWish: boolean = wishTypeStore.allCount === 1
    const isNewEvent = eventRecord.hasGeneratedId
    if (onlyOneWish && isNewEvent) {
        const eventClone = eventRecord.copy({
            name: wishTypeStore.first.name,
            frontendId: null,
            wishTypeId: wishTypeStore.first.id,
            note: '',
            userId: userId,
        })
        if (rowKey === 'user_id')
            eventClone.set({ resourceId: userId })
        source.eventStore.add([eventClone], true)
        source.eventStore.remove(eventRecord, true)
        return true
    }
    return false
}

export const sortJustAlphabetically = (a: ScheduleItemModelByUser, b: ScheduleItemModelByUser) => {
    if (a.eventType === CONSTS.shift && b.eventType === CONSTS.wish)
        return 0;

    const startA = DateHelper.format(a.startDate, "YYYY-MM-DD")
    const startB = DateHelper.format(b.startDate, "YYYY-MM-DD")
    if (startA !== startB)
        return 0;
    if (a.name.toLowerCase() == b.name.toLowerCase())
        return 0;
    if (a.name.toLowerCase() < b.name.toLowerCase())
        return -1;
    if (a.name.toLowerCase() > b.name.toLowerCase())
        return 1;
}

export const userCanSetDateButNotTime = (scheduler: Scheduler): boolean => {
    return scheduler.viewPreset.mainUnit === "day" && scheduler.fillTicks
}